@@ -7,6 +7,7 @@ from infi.pyutils.decorators import wraps
 CHECK_CONDITIONS_NOT_WORTH_RETRY = [
     ('ILLEGAL_REQUEST', 'LOGICAL UNIT NOT SUPPORTED'),
     ('ILLEGAL_REQUEST', 'INVALID FIELD IN CDB'),
+    ('ILLEGAL_REQUEST', 'INVALID COMMAND OPERATION CODE'),
 ]
 
 SEC = 1000
@@ -18,6 +19,12 @@ logger = getLogger(__name__)
 class ScsiCommandFailed(Exception):
     pass
 
+class ScsiCheckConditionError(ScsiCommandFailed):
+    def __init__(self, sense_key, code_name):
+        super(ScsiCheckConditionError, self).__init__(sense_key, code_name)
+        self.sense_key = sense_key
+        self.code_name = code_name
+
 def func_logger(func):
     @wraps(func)
     def decorator(*args, **kwargs):
@@ -71,10 +78,9 @@ def check_for_scsi_errors(func):
                 (key, code) = (e.sense_obj.sense_key, e.sense_obj.additional_sense_code.code_name)
                 msg = "{} sg device {} got {} {}".format(getpid(), sg_device, key, code)
                 logger.warn(msg)
-                if (key, code) in CHECK_CONDITIONS_NOT_WORTH_RETRY:
-                    # give up
-                    break
                 counter -= 1
+                if (key, code) in CHECK_CONDITIONS_NOT_WORTH_RETRY or counter == 0:
+                    raise ScsiCheckConditionError(key, code)
             except (IOError, OSError, AsiOSError, AsiSCSIError), error:
                 msg = "{} sg device {} got unrecoverable error {} during {}"
                 logger.error(msg.format(getpid(), sg_device, error, cdb))
